﻿#include <windows.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#define WIDTH 10          // 地图宽度
#define HEIGHT 10         // 地图高度
#define MINE_COUNT 15     // 地雷数量
#define CELL_SIZE 32      // 单元格大小

HINSTANCE hInst;        // 实例句柄
HWND hwndMain;          // 主窗口句柄
HWND hwndTimer;         // 计时器句柄
HWND hwndResetButton;   // 重置按钮句柄

char map[HEIGHT][WIDTH];            // 地图数据
int revealed[HEIGHT][WIDTH];        // 是否揭示
int mines[HEIGHT][WIDTH];           // 是否是地雷
int flags[HEIGHT][WIDTH];           // 是否标记为旗子
int gameOver = 0;                   // 游戏结束标志
int win = 0;                        // 胜利标志
int startTime = 0;                  // 开始时间
int elapsedTime = 0;                // 经过的时间

// 初始化地图
void initMap() {
    for (int i = 0; i < HEIGHT; i++) {
        for (int j = 0; j < WIDTH; j++) {
            map[i][j] = '.';          // 初始化地图为空
            revealed[i][j] = 0;       // 初始化未揭示
            mines[i][j] = 0;          // 初始化无地雷
            flags[i][j] = 0;          // 初始化无旗子
        }
    }

    srand(time(NULL));              // 随机数种子
    int count = 0;
    while (count < MINE_COUNT) {    // 布置地雷
        int x = rand() % WIDTH;
        int y = rand() % HEIGHT;
        if (!mines[y][x]) {
            mines[y][x] = 1;        // 放置地雷
            count++;
        }
    }
}

// 计算周围地雷数量
int countMines(int x, int y) {
    int count = 0;
    for (int i = -1; i <= 1; i++) {
        for (int j = -1; j <= 1; j++) {
            int nx = x + j;
            int ny = y + i;
            if (nx >= 0 && nx < WIDTH && ny >= 0 && ny < HEIGHT) {
                count += mines[ny][nx];   // 统计周围地雷数量
            }
        }
    }
    return count;
}

// 揭示单元格
void reveal(int x, int y) {
    if (x < 0 || x >= WIDTH || y < 0 || y >= HEIGHT || revealed[y][x] || flags[y][x]) {
        return;                         // 检查边界和状态
    }
    revealed[y][x] = 1;                 // 标记为已揭示
    if (mines[y][x]) {
        gameOver = 1;                     // 触发地雷，游戏结束
        return;
    }
    int mineCount = countMines(x, y);   // 计算周围地雷数量
    if (mineCount == 0) {
        for (int i = -1; i <= 1; i++) {
            for (int j = -1; j <= 1; j++) {
                reveal(x + j, y + i);     // 递归揭示相邻单元格
            }
        }
    }
}

// 检查是否胜利
int checkWin() {
    for (int i = 0; i < HEIGHT; i++) {
        for (int j = 0; j < WIDTH; j++) {
            if ((!mines[i][j] && !revealed[i][j]) || (mines[i][j] && !flags[i][j])) {
                return 0;                 // 检查是否有未揭示或未标记的地雷
            }
        }
    }
    return 1;                             // 全部揭示或标记正确，胜利
}

// 启动计时器
void startTimer() {
    startTime = GetTickCount();           // 获取当前时间戳
    SetTimer(hwndMain, 1, 1000, NULL);    // 设置定时器，每秒触发一次
}

// 停止计时器
void stopTimer() {
    KillTimer(hwndMain, 1);             // 停止定时器
}

// 重置游戏
void resetGame() {
    gameOver = 0;                       // 重置游戏结束标志
    win = 0;                            // 重置胜利标志
    initMap();                          // 初始化地图
    startTimer();                       // 启动计时器
    InvalidateRect(hwndMain, NULL, TRUE);// 刷新窗口
}

// 窗口过程函数
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
    switch (msg) {
    case WM_CREATE:
        hwndResetButton = CreateWindow(
            "BUTTON", "重置",
            WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON,
            8, 8, 64, 24,
            hwnd,
            (HMENU)1,
            ((LPCREATESTRUCT)lParam)->hInstance,
            NULL
        );
        initMap();                      // 初始化地图
        startTimer();                   // 启动计时器
        break;

    case WM_COMMAND:
        if (LOWORD(wParam) == 1) {      // 重置按钮点击事件
            resetGame();
        }
        break;

    case WM_TIMER:
        elapsedTime = (GetTickCount() - startTime) / 1000; // 计算经过时间
        InvalidateRect(hwndMain, NULL, TRUE);               // 刷新窗口
        break;

    case WM_LBUTTONDOWN: {
        if (gameOver || win) {
            PostQuitMessage(0);           // 游戏结束或胜利后退出
            break;
        }
        int x = (LOWORD(lParam) - 8) / CELL_SIZE;
        int y = (HIWORD(lParam) - 40) / CELL_SIZE;
        if (x >= 0 && x < WIDTH && y >= 0 && y < HEIGHT) {
            if (!flags[y][x]) {
                reveal(x, y);             // 揭示单元格
                if (checkWin()) {
                    win = 1;                // 检查是否胜利
                    stopTimer();            // 停止计时器
                } else if (gameOver) {
                    stopTimer();            // 停止计时器
                }
                InvalidateRect(hwndMain, NULL, TRUE); // 刷新窗口
            }
        }
        break;
    }

    case WM_RBUTTONDOWN: {
        if (gameOver || win) {
            PostQuitMessage(0);           // 游戏结束或胜利后退出
            break;
        }
        int x = (LOWORD(lParam) - 8) / CELL_SIZE;
        int y = (HIWORD(lParam) - 40) / CELL_SIZE;
        if (x >= 0 && x < WIDTH && y >= 0 && y < HEIGHT) {
            if (!revealed[y][x]) {
                flags[y][x] = !flags[y][x]; // 切换旗子标记
                InvalidateRect(hwndMain, NULL, TRUE); // 刷新窗口
            }
        }
        break;
    }

    case WM_PAINT: {
        PAINTSTRUCT ps;
        HDC hdc = BeginPaint(hwnd, &ps);

        HBRUSH brush = CreateSolidBrush(RGB(255, 255, 255));
        FillRect(hdc, &ps.rcPaint, brush);
        DeleteObject(brush);

        HPEN pen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
        SelectObject(hdc, pen);

        // 绘制单元格
        for (int i = 0; i < HEIGHT; i++) {
            for (int j = 0; j < WIDTH; j++) {
                RECT rect;
                rect.left = j * CELL_SIZE + 8;
                rect.top = i * CELL_SIZE + 40;
                rect.right = (j + 1) * CELL_SIZE + 8;
                rect.bottom = (i + 1) * CELL_SIZE + 40;

                if (revealed[i][j]) {
                    if (mines[i][j]) {
                        SetTextColor(hdc, RGB(255, 0, 0));
                        TextOut(hdc, rect.left + CELL_SIZE / 4, rect.top + CELL_SIZE / 4, "*", 1);
                    } else {
                        int mineCount = countMines(j, i);
                        if (mineCount > 0) {
                            char buffer[2];
                            sprintf(buffer, "%d", mineCount);
                            SetTextColor(hdc, RGB(0, 0, 255));
                            TextOut(hdc, rect.left + CELL_SIZE / 4, rect.top + CELL_SIZE / 4, buffer, 1);
                        }
                    }
                } else if (flags[i][j]) {
                    SetTextColor(hdc, RGB(255, 0, 0));
                    TextOut(hdc, rect.left + CELL_SIZE / 4, rect.top + CELL_SIZE / 4, "F", 1);
                } else {
                    Rectangle(hdc, rect.left, rect.top, rect.right, rect.bottom);
                }
            }
        }

        // 绘制计时器
        char timeStr[10];
        sprintf(timeStr, "Time: %03d", elapsedTime);
        SetTextColor(hdc, RGB(0, 0, 0));
        TextOut(hdc, 8, 8, timeStr, strlen(timeStr));

        // 绘制状态消息
        if (gameOver) {
            SetTextColor(hdc, RGB(255, 0, 0));
            TextOut(hdc, 8, HEIGHT * CELL_SIZE + 48, "你踩到地雷了！游戏结束。", 21);
        } else if (win) {
            SetTextColor(hdc, RGB(0, 255, 0));
            TextOut(hdc, 8, HEIGHT * CELL_SIZE + 48, "恭喜你，成功排雷！", 18);
        }

        DeleteObject(pen);
        EndPaint(hwnd, &ps);
        break;
    }

    case WM_DESTROY:
        stopTimer();                      // 停止计时器
        PostQuitMessage(0);               // 发送退出消息
        break;

    default:
        return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

// 主函数
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    hInst = hInstance;

    const char* CLASS_NAME = "MinesweeperClass";

    WNDCLASS wc = { };
    wc.lpfnWndProc = WndProc;
    wc.hInstance = hInstance;
    wc.lpszClassName = CLASS_NAME;

    RegisterClass(&wc);

    hwndMain = CreateWindowEx(
        0,
        CLASS_NAME,
        "扫雷游戏",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT,
        WIDTH * CELL_SIZE + 24, HEIGHT * CELL_SIZE + 76,
        NULL,
        NULL,
        hInstance,
        NULL
    );

    if (hwndMain == NULL) {
        return 0;
    }

    ShowWindow(hwndMain, nCmdShow);

    MSG msg = { };
    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return 0;
}



